function mesh = rigidMotion(mesh, M)
%% This function applies a rigid body transformation to the mesh
% MESH = RIGIDMOTION(MESH, M) Moves the mesh according to the homogeneous
% matrices defined in M for every joint of the mesh.
%
% INPUT:
%  mesh: A mesh struct containing at least the fields
%        vertices (3xN)
%        skinningJoints (WxN)
%        skinningWeights  (WxN)
%        skel
%  M   : cell array containing homogenous transformation matrices for every
%        joint. These transformation matrices are simply applied to
%        transform every vertex assigned to the corresponding joint without
%        regarding the skeleton hierarchy.
% OUTPUT:
%  mesh: A mesh with updated vertices and skeleton, remaining fields are
%        copies of the input mesh. A boundingBox field is generated.
%
% This function is part of the MPI08 data set, to be obtained at 
% http://www.tnt.uni-hannover.de/project/MPI08_Database/
%
% Written by: Andreas Baak
% Revision:   1.0
% Date:       28.09.2010
%
% Copyright 2010, MPI Informatik, Germany, and Hannover University, Germany.
% All Rights Reserved Permission to use, copy, modify, and distribute this
% software and its documentation for any non-commercial purpose is hereby
% granted without fee, provided that the above copyright notice appear in
% all copies and that both that copyright notice and this permission
% notice appear in supporting documentation, and that the name of the
% author not be used in advertising or publicity pertaining to distribution
% of the software without specific, written prior permission.
% THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
if ~isfield(mesh, 'skel')
    error('rididMotion:NoSkeletonInMesh', 'The mesh has to be rigged with a skeleton to allow a transformation.');
elseif ~isfield(mesh, 'skinningWeights')
    error('rididMotion:NoSkinning', 'The vertices have to be assigned to at least one joint using skinning weights.');
elseif ~isfield(mesh, 'vertices')
    error('rididMotion:NoVertices', 'Mesh struct is missing the vertices field.');
end
if numel(M) ~= numel(mesh.skel.nodes)
    error('ridigMotion:DimMismatch', 'The number of transformation matrices %d does not match the number of joints in the skeleton %d', numel(M), numel(mesh.skel.nodes));
end
vertices = [mesh.vertices; ones(1, size(mesh.vertices, 2))]; % transform to homogeneous coordinates
skinningJoints = mesh.skinningJoints;
skinningWeights =  mesh.skinningWeights;
numVertices = size(vertices, 2);
nodes = mesh.skel.nodes;
numNodes = numel(nodes);
numSkinningWeights = size(skinningWeights, 1);
% Apply motion to vertices
for  i = 1 : numVertices
    b = zeros(4, 1);
    for n = 1:numSkinningWeights
        b = b + M{skinningJoints(n, i)} * vertices(:, i)* skinningWeights(n, i);
    end
    vertices(:, i) = b;
end
% update vertices
mesh.vertices = vertices(1:3,:);
% Apply motion to joints
for i = numNodes :-1:2
    Mi = M{nodes(i).parent};
    R = Mi(1:3, 1:3);
    if not(nodes(i).isBallJoint)
        nodes(i).direction = R*nodes(i).direction;
    end
    nodes(i).position = Mi*[nodes(i).position; 1];
    nodes(i).position = nodes(i).position(1:3);
    nodes(i).moment = cross(nodes(i).position, nodes(i).direction);
end
nodes(1).position = M{1}*[nodes(1).position; 1];
nodes(1).position = nodes(1).position(1:3);
mesh.skel.nodes = nodes;
mesh.boundingBox = computeBoundingBoxVertices(mesh.vertices);
end